package edu.lawrence.questions;

import java.sql.*;
import java.util.ArrayList;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

public class QuizDAO {
    // The constructor opens the database connection
    public QuizDAO() {
        // Load the database driver
        try {
            Class.forName("com.mysql.cj.jdbc.Driver");
        } catch (ClassNotFoundException ex) {
            System.out.println("Could not load database driver.");
        }
        // Open the connection and set up statements
        try {
            connection = DriverManager.getConnection("jdbc:mysql://localhost:3306/quiz?user=student&password=Cmsc250!");
            statement = connection.createStatement();
            subjectsStmt = connection.prepareStatement(subjectsSQL);
            questionStmt = connection.prepareStatement(questionSQL);
            quizIDStmt = connection.prepareStatement(quizIDSQL);
            insertStmt = connection.prepareStatement(insertSQL);
            insertQuizStmt = connection.prepareStatement(insertQuizSQL);
            updateStmt = connection.prepareStatement(updateSQL);
            deleteStmt = connection.prepareStatement(deleteSQL);
            deleteQuizStmt = connection.prepareStatement(deleteQuizSQL);
            deleteQuizQuestionsStmt = connection.prepareStatement(deleteQuizQuestionsSQL);
        } catch (SQLException ex) {
            System.out.println("Could not connect to database.");
            ex.printStackTrace();
        }

        subjects = FXCollections.observableArrayList();
        
        rebuildSubjects();
    }
    
    public ObservableList<Subject> getSubjects() { return subjects; }
    
    public void rebuildSubjects() {
        subjects.clear();
        try {
            subjectsStmt.setInt(1, currentQuiz);
            ResultSet resultSet = subjectsStmt.executeQuery();
            while (resultSet.next()) {
                int id = resultSet.getInt("id");
                String subject = resultSet.getString("subject");
                subjects.add(new Subject(id,subject));
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
    }
    
    public void addQuestion(Question question) {
        try {
            insertStmt.setString(1,question.getSubject());
            insertStmt.setString(2,question.getQuestion());
            insertStmt.setString(3,question.getChoices());
            insertStmt.setString(4,question.getAnswer());
            insertStmt.setInt(5,currentQuiz);
            insertStmt.execute();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        rebuildSubjects();
    }
    
    public Question getQuestion(int id) {
        Question result = null;
        try {
            questionStmt.setInt(1, id);
            ResultSet resultSet = questionStmt.executeQuery();
            if (resultSet.next()) {
                String subject = resultSet.getString("subject");
                String question = resultSet.getString("question");
                String choices = resultSet.getString("choices");
                String answer = resultSet.getString("answer");
                result = new Question(id,subject,question,choices,answer);
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return result;
    }
    
    public void updateQuestion(Question question) {
        try {
            updateStmt.setString(1,question.getSubject());
            updateStmt.setString(2,question.getQuestion());
            updateStmt.setString(3,question.getChoices());
            updateStmt.setString(4,question.getAnswer());
            updateStmt.setInt(5,question.getId());
            updateStmt.execute();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        rebuildSubjects();
    }
    
    public void removeQuestion(Subject subject) {
        try {
            deleteStmt.setInt(1, subject.getId());
            deleteStmt.execute();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        subjects.remove(subject);
    }
    
    public void addQuiz(String quizName) {
        try {
            insertQuizStmt.setString(1,quizName);
            insertQuizStmt.execute();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
    }
    
    public void setQuiz(String quizName) {
        try {
            quizIDStmt.setString(1, quizName);
            ResultSet result = quizIDStmt.executeQuery();
            if(result.next()) {
                currentQuiz = result.getInt(1);
            } else
                currentQuiz = 0;
        }catch (SQLException ex) {
            currentQuiz = 0;
            ex.printStackTrace();
        }
        if(currentQuiz == 0)
            return;
        rebuildSubjects();
    }
    
    public void removeCurrentQuiz() {
        if(currentQuiz != 0) {
            try {
                deleteQuizQuestionsStmt.setInt(1, currentQuiz);
                deleteQuizQuestionsStmt.execute();
                deleteQuizStmt.setInt(1, currentQuiz);
                deleteQuizStmt.execute();
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
        }
    }
    
    public ArrayList<String> getQuizNames() {
        ArrayList<String> results = new ArrayList<String>();
        try {
            ResultSet resultSet = statement.executeQuery("select title from quizzes");

            while (resultSet.next()) {
                results.add(resultSet.getString(1));
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return results;    
    }
    
    private int currentQuiz;
    private Connection connection;
    private Statement statement;
    private String subjectsSQL = "select id, subject from questions where quiz=?";
    private PreparedStatement subjectsStmt;
    private String questionSQL = "select subject,question,choices,answer from questions where id=?";
    private PreparedStatement questionStmt;
    private String quizIDSQL = "select id from quizzes where title=?";
    private PreparedStatement quizIDStmt;
    private String insertSQL = "insert into questions(subject,question,choices,answer,quiz) values(?,?,?,?,?)";
    private PreparedStatement insertStmt;
    private String insertQuizSQL = "insert into quizzes(title) values(?)";
    private PreparedStatement insertQuizStmt;
    private String updateSQL = "update questions set subject=?,question=?,choices=?,answer=? where id = ?";
    private PreparedStatement updateStmt;
    private String deleteSQL = "delete from questions where id=?";
    private PreparedStatement deleteStmt;
    private String deleteQuizSQL = "delete from quizzes where id=?";
    private PreparedStatement deleteQuizStmt;
    private String deleteQuizQuestionsSQL = "delete from questions where quiz=?";
    private PreparedStatement deleteQuizQuestionsStmt;
    private ObservableList<Subject> subjects;
}
