package edu.lawrence.questionsfx;

import edu.lawrence.questions.Question;
import edu.lawrence.questions.QuizDAO;
import edu.lawrence.questions.Subject;
import java.net.URL;
import java.util.ArrayList;
import java.util.Optional;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.ListView;
import javafx.scene.control.TextInputDialog;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;

public class PrimaryController implements Initializable {
    private QuizDAO dao;
    
    @FXML
    private ChoiceBox quizChoice;
    
    @FXML
    private ListView subjectsList;
    
    @FXML
    private void newQuestion(ActionEvent event) {
        Stage parent = (Stage) subjectsList.getScene().getWindow();

        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource("FXMLEditQuestionDialog.fxml"));
            Parent root = (Parent)loader.load();
            Scene scene = new Scene(root);
            Stage dialog = new Stage();
            dialog.setScene(scene);
            dialog.setTitle("Edit Question");
            dialog.initOwner(parent);
            dialog.initModality(Modality.WINDOW_MODAL);
            dialog.initStyle(StageStyle.UTILITY);
            dialog.setX(parent.getX() + parent.getWidth()/4);
            dialog.setY(parent.getY() + parent.getHeight()/3);

            FXMLEditQuestionDialogController controller = (FXMLEditQuestionDialogController) loader.getController();
            controller.setDAO(dao);
            controller.setQuestion(null);
            dialog.show();
        } catch(Exception ex) {
            System.out.println("Could not open dialog.");
            ex.printStackTrace();
        }
    }
    
    @FXML
    private void editQuestion(ActionEvent event) {
        Stage parent = (Stage) subjectsList.getScene().getWindow();
        Subject toEdit = (Subject) subjectsList.getSelectionModel().getSelectedItem();
        if(toEdit == null)
            return;
        Question question = dao.getQuestion(toEdit.getId());
        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource("FXMLEditQuestionDialog.fxml"));
            Parent root = (Parent)loader.load();
            Scene scene = new Scene(root);
            Stage dialog = new Stage();
            dialog.setScene(scene);
            dialog.setTitle("Edit Question");
            dialog.initOwner(parent);
            dialog.initModality(Modality.WINDOW_MODAL);
            dialog.initStyle(StageStyle.UTILITY);
            dialog.setX(parent.getX() + parent.getWidth()/4);
            dialog.setY(parent.getY() + parent.getHeight()/3);

            FXMLEditQuestionDialogController controller = (FXMLEditQuestionDialogController) loader.getController();
            controller.setDAO(dao);
            controller.setQuestion(question);
            dialog.show();
        } catch(Exception ex) {
            System.out.println("Could not open dialog.");
            ex.printStackTrace();
        }    
    }
    
    @FXML
    private void removeQuestion(ActionEvent event) {
        Subject toRemove = (Subject) subjectsList.getSelectionModel().getSelectedItem();
        dao.removeQuestion(toRemove);
    }
    
    @FXML
    private void quit(ActionEvent event) {
        Platform.exit();
    }
    
    @FXML
    private void newQuiz(ActionEvent event) {
        TextInputDialog dialog = new TextInputDialog();
        dialog.setHeaderText(null);
        dialog.setGraphic(null);
        dialog.setTitle("New Quiz");
        dialog.setContentText("Enter a quiz name:");

        Optional<String> result = dialog.showAndWait();
        if (result.isPresent()){
            String quiz = result.get();
            dao.addQuiz(quiz);
            quizChoice.getItems().clear();
            ArrayList<String> quizNames = dao.getQuizNames();
            quizChoice.getItems().addAll(quizNames);
            quizChoice.setValue(quiz);
            dao.setQuiz(quiz);
        }       
    }
    
    @FXML
    private void removeQuiz(ActionEvent event) {
        dao.removeCurrentQuiz();
        quizChoice.getItems().clear();
        ArrayList<String> quizNames = dao.getQuizNames();
        String quiz = quizNames.get(0);
        quizChoice.getItems().addAll(quizNames);
        quizChoice.setValue(quiz);
        dao.setQuiz(quiz);
    }
    
    private void changeQuiz(Object quiz) {
        if(quiz != null)
            dao.setQuiz(quiz.toString());
    }
    
    @Override
    public void initialize(URL url, ResourceBundle rb) {
        dao = new QuizDAO();

        ArrayList<String> quizNames = dao.getQuizNames();
        String quiz = quizNames.get(0);
        quizChoice.getItems().addAll(quizNames);
        quizChoice.setValue(quiz);
        quizChoice.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue)->changeQuiz(newValue));
        dao.setQuiz(quiz);
        subjectsList.setItems(dao.getSubjects());
    }    
    
}
